// Draw pixels demo

#include "defines.h"
#ifdef DEMO0
#include "quark.h"
#include "armlet.h"
#include "libutil.h"

CVGpu VGpu;                                      // create a virtual gpu object
static int SinusPhase;                           // current phase, used to animate sinus and color

// main game init, must return 1 if success
int InitGame(void)
{
  // init vgpu. although 3D not used in this demo, init must be done
  if (!VGpu.Init(CLIPZ_NEAR, CLIPZ_FAR, 60))     // init vgpu, Fov = 60 degree (2*PI = 360 degree)
    return 0;

  SinusPhase = 0;
  return 1;                                      // init success
}

// main game loop, time is passed, must return 0 to quit
int LoopGame(void)
{
  // use SinusPhase to define a custom color for frame
  // note: range of values must be 0-255
  int R = (SinusPhase >> 2) & 0xff;
  int G = (50 + (SinusPhase >> 3)) & 0xff;
  int B = (100 + (SinusPhase >> 4)) & 0xff;
  int Color = COLOR_RGB(R, G, B) | COLOR_I(255); // define color with maximum light intensity (255)

  for (int i=0; i<ANGLE2PI; i+=8)
  {
    int a = i*5;                                 // define angle to obtain 4 periods
    int b = i*7;                                 // define angle to obtain 3 periods

    int x = (SINUS(a + SinusPhase) * 110) >> TRIGO_SHIFT;   // define x amplitude
    int y = (COSINUS(b + SinusPhase) * 110) >> TRIGO_SHIFT; // define y amplitude

    // add offset to define some centered XY axis
    // note x=0, y=0 is located at top left of screen
    int xc = (SCR_X/2) + x;
    int yc = (SCR_Y/2) - y;

    VGpu.PutPixel(xc, yc, Color);    

    if (!i)                                      // draw text one time at first pixel position
      VGpu.PrintXY("Pixel Drawing Demo", xc - 80, yc);
  }
  VGpu.PrintFps();                               // display framerate
  
  SinusPhase += 3;                               // define new phase to animate screen
  return !ARMData->Mouse.LButtonDown;            // loop util click/tap on screen occur
}

// blit datas into video memory, and clear virtual screen in dark blue
void BlitScreen(void)
{
  VGpu.FlushFB(COLOR_RGB(0, 0, 64));
}

// called before to quit (click on screen occured, or escape pressed, or LoopGame return 0)
void EndGame(void)
{
  VGpu.Delete();                                 // free used memory
}
#endif // DEMOx
