// Draw lines demo

#include "defines.h"
#ifdef DEMO1
#include "quark.h"
#include "armlet.h"

CVGpu VGpu;                                      // create a virtual gpu object
static int SinusPhase;                           // current phase, used to animate sinus and color

// main game init, must return 1 if success
int InitGame(void)
{
  if (!VGpu.Init(CLIPZ_NEAR, CLIPZ_FAR, 60))     // init vgpu
    return 0;

  SinusPhase = 0;
  return 1;                                      // init success
}

#define SQR_CNT 40                               // number of square to draw

// main game loop, time is passed, must return 0 to quit
int LoopGame(void)
{
  // use SinusPhase to define a custom color for frame
  int R = (SinusPhase >> 2) & 0xff;
  int G = (50 + (SinusPhase >> 4)) & 0xff;
  int B = (100 + (SinusPhase >> 4)) & 0xff;
  VGpu.DrawCfg.LineColor = COLOR_RGB(R, G, B);   // define base color

  int a = SinusPhase;
  int Intensity = 255;                           // line color intensity
  int IStep = 200/SQR_CNT;                       // step to modify intensity

  for (int i=0; i<SQR_CNT; i++)
  {
    int S[4][3];                                 // 4 xyz position used to draw square using lines
    int r = 200 - i*5;                           // square radius

    int b = a;
    for (int j=0; j<4; j++)
    {
      S[j][0] = (SCR_X/2) + ((SINUS(b) * r) >> TRIGO_SHIFT);
      S[j][1] = (SCR_Y/2) - ((COSINUS(b) * r) >> TRIGO_SHIFT);

      // Z depht. note: when working in immediate mode with screen coordinates (lines and spriteZb),
      // z value must be positive in range [-CLIPZ_NEAR..-CLIPZ_FAR].
      S[j][2] = -CLIPZ_NEAR + i;
      b += ANGLE2PI/4;
    }
    a += ANGLE2PI/50;                            // rotation

    // draw square using lines
    VGpu.DrawCfg.LineIntensity = Intensity;      // set line drawing intensity
    Intensity -= IStep;                          // change intensity

    VGpu.DrawLine(S[0], S[1]);
    VGpu.DrawLine(S[1], S[2]);
    VGpu.DrawLine(S[2], S[3]);
    VGpu.DrawLine(S[3], S[0]);
  }

  VGpu.PrintXY("Lines Drawing Demo", 0, 16);
  VGpu.PrintFps();                               // display framerate
  
  SinusPhase += 10;                              // define new phase to animate screen
  return !ARMData->Mouse.LButtonDown;            // loop util click/tap on screen occur
}

// blit datas into video memory, and clear virtual screen in dark blue
void BlitScreen(void)
{
  VGpu.FlushFB(COLOR_RGB(0, 0, 64));
}

// called before to quit (click on screen occured, or escape pressed, or LoopGame return 0)
void EndGame(void)
{
  VGpu.Delete();                                 // free used memory
}
#endif // DEMOx