// High polygon scene. this can be used as machine benchmark.
// note that the score ar lower than on ppc having same speed arm cpu. This is mainly due
// to fact that screen size is bigger on palm (320*320 vx 240*320) so blit time take more time.

#include "defines.h"
#ifdef DEMO14

#include "libutil.h"
#include "quark.h"
#include "armlet.h"
#include "meshdata.h"
#include "import.h"

CVGpu VGpu;                                      // gpu object
CTextureList TextureList;                        // use a list of textures
CImport Import;                                  // CImport object used to load mesh + textures from file

#define NB_TREE 20                               // number of tree to put on landscape
#define TEX_COUNT 3                              // 3 texture for scene

CTriangleList TreeTL;                            // triangle list for tree
CTriangleList LandTL;                            // triangle list for landscape

// trees
typedef struct
{
  int Pos[3];                                    // position on landscape
  int Scale;                                     // scale
} TTree;

TTree Forest[NB_TREE];                           // some tree to define a 'very small' forest
int TreeBBox[8][3];                              // bounding box for tree

// camera
typedef struct
{
  short Pos[3];
  short Angle;
} TCamPos;

TCamPos CamPath[ANGLE2PI/16];                    // path followed by camera

int PathIndex;                                   // used to define a camera circular path on landscape
int CamY;                                        // altitude of camera

// benchmark
char BenchStat[100];                             // max number of vertex / faces message
char BenchResult0[100];                          // result of benchmark (average fps)
char BenchResult1[100];                          // result of benchmark (score)
unsigned int BenchTimeSum;                       // bench time elapsed

// return object min/max position of vertex
void GetMinMax(CTriangleList *Tl, int *Min, int *Max)
{
  Min[0] = Min[1] = Min[2] = 32767;
  Max[0] = Max[1] = Max[2] = -32767;

  for (int i=0; i<Tl->VertexListSize; i++)
  {
    int Pos[3];
    Tl->GetXYZ(i, Pos);

    if (Pos[0] < Min[0]) Min[0] = Pos[0];
    if (Pos[1] < Min[1]) Min[1] = Pos[1];
    if (Pos[2] < Min[2]) Min[2] = Pos[2];
    if (Pos[0] > Max[0]) Max[0] = Pos[0];
    if (Pos[1] > Max[1]) Max[1] = Pos[1];
    if (Pos[2] > Max[2]) Max[2] = Pos[2];
  }
}

// return altitude for a given position on landscape
// note: use nearest vertex found near xz position, not accurate but enough to place a tree
int Altitude(CTriangleList *Tl, int x, int z)
{
  int Miny = 32768;
  int Mind = 0x7fffffff;
  for (int i=0; i<Tl->VertexListSize; i++)
  {
    int Index = i*Tl->VertexStride;
    int dx = Tl->VertexList[Index] - x;
    int dz = Tl->VertexList[Index+2] - z;
    int d2 = dx*dx + dz*dz;
    if (d2 < Mind)
    {
      Mind = d2;
      Miny = Tl->VertexList[Index+1];
    }
  }
  return Miny;
}

// init a bounding box, this is use to know if a tree is in view frustrum,
// this avoid to draw things that are not visibles.
// box is defined with 8 vectors (3*int each) and must be defined as follow:
// note: 0 = LMin, 7 = LMax
//           4 ---- 5
//          /|     /|
//         / |    / |
//        6 ---- 7  |
// +y     |  |   |  |
// | -z   |  0 --|- 1
// |/     | /    | /
// O--+x  |/     |/
//        2 ---- 3
void InitBoundingBox(int *LMin, int *LMax, int *Box)
{
  // bottom
  Box[0]=LMin[0];   Box[1]=LMin[1];   Box[2]=LMin[2];      // 0
  Box[3]=LMax[0];   Box[4]=LMin[1];   Box[5]=LMin[2];      // 1
  Box[6]=LMin[0];   Box[7]=LMin[1];   Box[8]=LMax[2];      // 2
  Box[9]=LMax[0];   Box[10]=LMin[1];  Box[11]=LMax[2];     // 3

  // top
  Box[12]=LMin[0];  Box[13]=LMax[1];  Box[14]=LMin[2];     // 4
  Box[15]=LMax[0];  Box[16]=LMax[1];  Box[17]=LMin[2];     // 5
  Box[18]=LMin[0];  Box[19]=LMax[1];  Box[20]=LMax[2];     // 6
  Box[21]=LMax[0];  Box[22]=LMax[1];  Box[23]=LMax[2];     // 7
}

// main game init
int InitGame(void)
{
  int i;
  if (!VGpu.Init(CLIPZ_NEAR, CLIPZ_FAR, 60))     // init vgpu, Fov = 60 degree (2*PI = 360 degree)
    return 0;

  if (!TextureList.Alloc(TEX_COUNT))             // alloc list for 2 textures
    return 0;

  // ----------------
  // init tree object

  if (!Import.InitTL("Mesh/Tree.msh", &TreeTL, VF_XYZ|VF_I, IF_ABC|IF_UV, 255, &TextureList, 1))
    return 0;

  // init tree bounding box
  int TMin[3];
  int TMax[3];
  GetMinMax(&TreeTL, TMin, TMax);
  InitBoundingBox(TMin, TMax, (int *)TreeBBox);

  // set keyed alpha mode for tree leaf texture + 2 sided
  CTexture *T = TextureList.GetTexture("Tga24/Feuil2.tga");
  T->DefColorKey(COLOR_RGB(0, 0, 0));            // note: may be skipped if texture key is 0 in image (the case here)
  T->RasterMode = RM_KEYED;
  T->Vis2Sided = 1;

  // --------------
  // init landscape

  if (!Import.InitTL("Mesh/Land.msh", &LandTL, VF_XYZ|VF_I, IF_ABC|IF_UV, 255, &TextureList, 1))
    return 0;

  // get landscape size, used to place trees on landscape
  int LMin[3];
  int LMax[3];
  GetMinMax(&LandTL, LMin, LMax);                // get size of landscape

  // available space area on landscape
  int LSize[3];
  LSize[0] = LMax[0] - LMin[0];                  // x size
  LSize[2] = LMax[2] - LMin[2];                  // z size (up is y)

  // -----------------------------------------------------------------------------
  // define camera path. to do that we use a trigonometric function (shape of '8')

  // set camera move radius
  int rMin;
  if (LSize[0] < LSize[2])
    rMin = LSize[0];
  else
    rMin = LSize[2];
  int CamRadius = (rMin*60)/200;                 // = (Size / 2)*0.6

  for (i=0; i<ANGLE2PI/16; i++)
  {
    int a = i*16;
    int s = SINUS(2*a);
    int c = COSINUS(a);
    int x = (s*CamRadius) >> TRIGO_SHIFT;
    int z = (c*CamRadius) >> TRIGO_SHIFT;
    CamPath[i].Pos[0] = x;
    CamPath[i].Pos[2] = z;
    CamPath[i].Pos[1] = Altitude(&LandTL, x, z) + 140;

    // get looking direction on curve
    // tan(LookAngle) = c'/s' = -s/2*cos(2*s)
    int s2 = SINUS(a);
    int c2 = COSINUS(2*a);
    CamPath[i].Angle = (short)(Atan2i(-s2, 2*c2) + ANGLE2PI/2);
  }

  // initial altitude and phase
  CamY = CamPath[0].Pos[1];
  PathIndex = 0;

  // ------------------------------------------------------------
  // init some random positions of trees on landscape. 
  // take care than tree are not too near path and themself

  int RandPos[3];
  int Border[3];

  // define some borders to avoid to put trees too near of borders
  Border[0] = LSize[0] / 12;
  Border[2] = LSize[2] / 12;

  // remove borders to delta
  LSize[0] -= Border[0]*2;
  LSize[2] -= Border[2]*2;

  // define random position/scale/rotation
  for (i=0; i<NB_TREE; i++)
  {
    // define a random scale and rotation
    Forest[i].Scale = 200 + (rand() % 500);

    // init tree position
    for (int t=0; t<500; t++)                    // try 500 random positions
    {
      RandPos[0] = LMin[0] + Border[0] + (rand() % (LSize[0] - Border[0]*2));
      RandPos[2] = LMin[2] + Border[2] + (rand() % (LSize[2] - Border[2]*2));
    
      // check if not too near of another tree
      int j, D2, AB[3];
      for (j=0; j<i; j++)
      {
        AB[0] = Forest[j].Pos[0] - RandPos[0];
        AB[2] = Forest[j].Pos[2] - RandPos[2];
        D2 = AB[0]*AB[0] + AB[2]*AB[2];
        if (D2 < 160*160)                        // 400 = min allowed distance
          break;                                 // to near, find new pos
      }
      if (j != i)                                // previous test failed
        continue;

      // check if not too near of camera path
      for (j=0; j<ANGLE2PI/16; j++)
      {
        AB[0] = CamPath[j].Pos[0] - RandPos[0];
        AB[2] = CamPath[j].Pos[2] - RandPos[2];
        D2 = AB[0]*AB[0] + AB[2]*AB[2];
        if (D2 < 130*130)                        // 250 = min allowed distance
          break;                                 // to near, find new pos
      }
      if (j == ANGLE2PI/16)                      // previous test succeded
        break;
    }

    // store position
    Forest[i].Pos[0] = RandPos[0];
    Forest[i].Pos[2] = RandPos[2];

    // get altitude at this position
    Forest[i].Pos[1] = Altitude(&LandTL, RandPos[0], RandPos[2]);
  }

  // reserve some working memory for internal work
  int vxmax = MAX(TreeTL.VertexListSize, LandTL.VertexListSize);
  if (!VGpu.SetWorkMemory(vxmax, 0))
    return 0;

  // prepare bench string
  int VxSum = TreeTL.VertexListSize*NB_TREE + LandTL.VertexListSize; // max number of vertex / scene if all is visible
  int TSum = TreeTL.IndexListSize*NB_TREE + LandTL.IndexListSize;    // max number of tringles / scene if all is visible
  itoa(VxSum, BenchStat);
  strcat(BenchStat, " Vertex, ");
  itoa(TSum, &BenchStat[strlen(BenchStat)]);
  strcat(BenchStat, " Faces");

  // change max allowed error for perspective correction. this allow, if low correction required, to draw face using
  // not corrected drawing code that is more fast. the range is about [200..1500]
  //   200: low error allowed
  //  1500: big error allowed
  // default value is 0: correction is always applyed, even if not required
  VGpu.ZcErrorMax = 1500;

  // definie a mininal size to allow texturing. This increase a bit drawing speed.
  // very small or stretched triangles will be drawn in untextured mode.
  VGpu.DrawCfg.TexTrigLevel = 64;                // note: 0 to disable.

  // reset time counters
  BenchTimeSum = 0;
  ARMData->Timedtms = 0;

  return 1;                                      // init success
}

// -----------------------------------------------------
// main game loop, time is passed, must return 0 to quit
int LoopGame(void)
{
  if (VGpu.FrameCounter)                         // avoid dt of frame 0 (delay from previous frame not valid)
    BenchTimeSum += ARMData->Timedtms;

  // -------------------
  // set camera position
  PathIndex++;
  if (PathIndex >= (ANGLE2PI/16))                // loop in index
    PathIndex = 0;

  TCamPos *Cp = &CamPath[PathIndex];
  CamY = (CamY*15 + Cp->Pos[1]) >> 4;            // smooth altitude

  CMatrix CamMat;
  CamMat.LoadIdentity();
  CamMat.RotateY(-Cp->Angle);
  CamMat.Translate(-Cp->Pos[0], -CamY, -Cp->Pos[2]);

  // ----------
  // draw trees
  VGpu.DrawCfg.ZCorrection = 0;                  // z correction not required for trees (small faces)
  CMatrix TreeMat;
  VGpu.ViewMatrix = &TreeMat;

  for (int i=0; i<NB_TREE; i++)
  {
    TreeMat.Load(&CamMat);
    TreeMat.Translate(Forest[i].Pos[0], Forest[i].Pos[1], Forest[i].Pos[2]);
    TreeMat.Scale(Forest[i].Scale);

    if (VGpu.CrossFrustrum((int *)TreeBBox))     // test if bbox of tree is visible
      VGpu.DrawTriangleList(&TreeTL, &TextureList);
  }

  // --------------
  // draw landscape
  VGpu.DrawCfg.ZCorrection = 1;                  // z correction required for landscape
  VGpu.ViewMatrix = &CamMat;                     // set VGpu view matrix. note: no use to multiply with landscape matrix (= identity)
  VGpu.DrawTriangleList(&LandTL, &TextureList);  // note: return value of DrawTriangleList ignored here

  // text infos
  if (VGpu.FrameCounter < 150)
  {
    VGpu.PrintXY("Forest benchmark", 0, 15);
    VGpu.PrintXY(BenchStat, 0, 30);
  }
  else
  if (VGpu.FrameCounter == 500)
  {
    int Score = 10000000/BenchTimeSum;
    int AvgFps = 500000/BenchTimeSum;
    strcpy(BenchResult0, "Average fps = ");
    itoa(AvgFps, &BenchResult0[strlen(BenchResult0)]);
    strcpy(BenchResult1, "Bench score = ");
    itoa(Score, &BenchResult1[strlen(BenchResult1)]);
  }
  else
  if (VGpu.FrameCounter > 500)
  {
    VGpu.PrintXY(BenchResult0, 0, 15);
    VGpu.PrintXY(BenchResult1, 0, 30);
  }

  VGpu.PrintFps();                               // display framerate
  return !ARMData->Mouse.LButtonDown;            // loop util click/tap on screen occur
}

// blit datas into video memory, and clear virtual screen in dark blue
void BlitScreen(void)
{
  VGpu.FlushFB(COLOR_RGB(0, 0, 64));
}

// free memory
void EndGame(void)
{
  TreeTL.Delete();
  LandTL.Delete();
  TextureList.Delete();
  VGpu.Delete();
}
#endif // DEMOx
