// Draw a single triangle using direct drawing with screen coordinates

#include "defines.h"
#ifdef DEMO5
#include "libutil.h"
#include "quark.h"
#include "tgafile.h"
#include "armlet.h"

CVGpu VGpu;                                      // create a virtual gpu object
CTexture Texture;                                // a texture to map on triangle

// main init
int InitGame(void)
{
  if (!VGpu.Init(CLIPZ_NEAR, CLIPZ_FAR, 70))     // init vgpu, Fov = 70 degree (2*PI = 360 degree)
    return 0;

  // load texture from files
  char Path[80];
  int x, y;
  unsigned char *Pixels;

  strcpy(Path, "Tga24/");                        // tga images are stored in Data folder
  strcat(Path, "FrogW.tga");

  if (!LoadTga(Path, &x, &y, &Pixels))           // load a tga file to define texture
    return 0;

  // init the texture.
  // note: 
  //  - Init return 1 if size x and y are a power of 2 and in 16 to 128 range. This is
  //    required only to draw triangles. Can be ignored to draw sprites (no size constraints in this case)
  if (!Texture.Init(Pixels, x, y, Path))
    return 0;
  return 1;                                      // init success
}

// main loop, time is passed, must return 0 to quit
int LoopGame(void)
{
  // need 3 vertex to draw a triangle
  TVertex A, B, C;

  // ----------------------------
  // draw one untextured triangle

  // define vertex values for screen
  // texture coordinates do no need to be initialized
  A.x = 80;
  A.y = 50;
  A.z = 100;                                     // define a valid Z position between -CLIPZ_NEAR and -CLIPZ_FAR
  A.m = 0;                                       // position in frustrum
  A.i = 120;                                     // light intensity

  B.x = 50;
  B.y = 150;
  B.z = 100;
  B.m = 0;
  B.i = 255;

  C.x = 120;
  C.y = 250;
  C.z = 100;
  C.m = 0;
  C.i = 0;

  // draw opaque yellow untextured triangle
  VGpu.DrawTriangleU(&A, &B, &C, COLOR_RGB(255, 255, 0), 0);

  // --------------------------
  // draw one textured triangle

  // define vertex values for screen
  A.x = 180;
  A.y = 50;
  A.z = 100;                                     // define a valid Z position between -CLIPZ_NEAR and -CLIPZ_FAR
  A.m = 0;
  A.i = 100;                                     // light intensity
  A.u = 0;
  A.v = UV_MUL;                                  // fixed point equivalent to 1.0f

  B.x = 150;
  B.y = 180;
  B.z = 100;
  B.m = 0;
  B.i = 200;
  B.u = 0;
  B.v = 0;

  C.x = 230;
  C.y = 200;
  C.z = 100;
  C.m = 0;
  C.i = 255;
  C.u = UV_MUL;
  C.v = 0;

  VGpu.DrawTriangleT(&A, &B, &C, &Texture);

  VGpu.PrintXY("Direct draw triangle demo", 0, 16);
  VGpu.PrintFps();                               // display framerate
  return !ARMData->Mouse.LButtonDown;            // loop util click/tap on screen occur
}

// blit datas into video memory, and clear virtual screen in dark blue
void BlitScreen(void)
{
  VGpu.FlushFB(COLOR_RGB(0, 0, 64));
}

// called before to quit (click on screen occured, or escape pressed, or LoopGame return 0)
void EndGame(void)
{
  free(Texture.Texels);                          // free texture memory
  VGpu.Delete();                                 // free VGpu memory
}
#endif // DEMOx
